local Emphasize = Grid:GetModule("GridStatus"):GetModule("GridStatusEmphasize")
local QuickHealth = LibStub and LibStub("LibQuickHealth-2.0", true) -- don't error if not found
local GridStatusEmphasize_Health = Emphasize:NewModule("GridStatusEmphasize_Health", "AceEvent-2.0")

local GridRoster = Grid:GetModule("GridRoster")
local GridFrame = Grid:GetModule("GridFrame")

-- upvalues
local UnitHealth = QuickHealth and QuickHealth.UnitHealth or UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitGUID = UnitGUID
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitIsConnected = UnitIsConnected
local UnitIsCharmed = UnitIsCharmed
local UnitIsEnemy = UnitIsEnemy

GridStatusEmphasize_Health.defaultDB = {
	range = true,
	always_show_lowhp_pc = 0.5,
	emphasize_lowhp_pc = 0.3,
}

local options = {
	["lowhp_pc"] = {
		order = 1,
		type = "range",
		name = "Never fade out threshold",
		desc = "Never fade out units under HP %(0 to disable)",
		get = function () return GridStatusEmphasize_Health.db.profile.always_show_lowhp_pc*100 end,
		set = function (v)
				GridStatusEmphasize_Health.db.profile.always_show_lowhp_pc = v/100
				GridStatusEmphasize_Health:UpdateAll()
			end,
		min = 0,
		max = 100,
		step = 1,
	},
	["emphasize_lowhp_pc"] = {
		order = 2,
		type = "range",
		name = "Emphasize threshold",
		desc = "Emphasize units under HP % (0 to disable)",
		get = function () return GridStatusEmphasize_Health.db.profile.emphasize_lowhp_pc*100 end,
		set = function (v)
				GridStatusEmphasize_Health.db.profile.emphasize_lowhp_pc = v/100
				GridStatusEmphasize_Health:EnableDisable()
			end,
		min = 0,
		max = 100,
		step = 1,
	},
	["range"] = {
		type = "toggle",
		name = "Ranger filter",
		desc = "Ignore units out of range",
		get = function () return GridStatusEmphasize_Health.db.profile.range end,
		set = function (v)
				GridStatusEmphasize_Health.db.profile.range = v
				GridStatusEmphasize_Health:EnableDisable()
			end,
	},
}

local settings

function GridStatusEmphasize_Health:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterOptions("Health", options)
	settings = self.db.profile

	self:EnableDisable()
end

function GridStatusEmphasize_Health:EnableDisable()
	if settings.always_show_lowhp_pc ~= 0 or settings.emphasize_lowhp_pc ~= 0 then
		if QuickHealth then
			QuickHealth.RegisterCallback(self, "UnitHealthUpdated", "UnitHealthUpdated")
		else
			self:RegisterEvent("UNIT_HEALTH", "UpdateUnit")
		end
		self:RegisterEvent("UNIT_HEALTH_MAX", "UpdateUnit")
	elseif self:IsEventRegistered("UNIT_HEALTH_MAX") then
		if QuickHealth then
			QuickHealth.UnregisterCallback(self, "UnitHealthUpdated")
		else
			self:UnregisterEvent("UNIT_HEALTH", "UpdateUnit")
		end
		self:UnregisterEvent("UNIT_HEALTH_MAX", "UpdateUnit")
	end	

	self:UpdateAll()
end

function GridStatusEmphasize_Health:UpdateAll()
	Emphasize:Pause()
	for guid, unitid in GridRoster:IterateRoster() do
		self:UpdateUnit(unitid, guid)
	end
	Emphasize:Resume()
end

function GridStatusEmphasize_Health:UnitHealthUpdated(event, unitid, health, healthmax)
	local guid = UnitGUID(unitid)
	
	if GridRoster:IsGUIDInRaid(guid) and not UnitIsDeadOrGhost(unitid) and UnitIsConnected(unitid) and not (UnitIsCharmed(unitid) and UnitIsEnemy("player", unitid)) and (not settings.range or GridFrame:UnitInRange(unitid, 40)) then
		local hp = health / healthmax
		
		if hp < settings.emphasize_lowhp_pc then
			Emphasize:TempEmphasizeUnit(guid, "lowhp_emphasize")
		else
			Emphasize:TempDeemphasizeUnit(guid, "lowhp_emphasize")
		end
		
		if hp <  settings.always_show_lowhp_pc then
			Emphasize:StaticEmphasizeUnit(guid, "lowhp_static")
		else
			Emphasize:StaticDeemphasizeUnit(guid, "lowhp_static")
		end		
	end
end

function GridStatusEmphasize_Health:UpdateUnit(unitid)
	self:UnitHealthUpdated("UnitHealthUpdated", unitid, UnitHealth(unitid), UnitHealthMax(unitid))
end
